@@ -1,40 +1,68 @@
 import json
 import pathlib
-from typing import Optional, Dict, Any, List, Union
+import glob
+import re
+import psutil
+from typing import Optional, Dict, Any, List, Union, Iterator, Tuple
 from pydantic import BaseModel
 from fastapi import Security, HTTPException
 from .app import app
 from .security import User, process_user_claim
-from ..lib.qemu import write_qemu_service_file, create_qemu_string, verify_qemu_resources, qemu_img
+from ..lib.qemu import (
+	write_qemu_service_file,
+	create_qemu_string,
+	verify_qemu_resources,
+	qemu_img,
+	get_diskimage_information
+)
 from ..lib.helpers.dupedict import DupeDict, JSON
 from ..lib.helpers.syscalls import SysCommand
+from ..lib.helpers.logger import log
+# from ..lib.networking.interfaces import curate_interfaces
+
+class NetworkInterfaces(BaseModel):
+	type: str
+	name: str
+	bridge: Optional[str]
+	namespace: Optional[Union[str, bool]]
+	attach: Optional[bool]
+	veth_pair: Optional[str]
+	mac: Optional[str]
 
 class NewVirtualMachine(BaseModel):
-	namespace: Optional[str] = None
-	cpu: str = "host"
-	enable_kvm: bool = True
-	machine: str = "q35,accel=kvm"
-	devices: List[str] = ["intel-iommu"]
-	memory: int = 8192 # TODO: Make this value dynamic by reading machine total memory / 4 or something
-	drives: List[str] = [
-		"if=pflash,format=raw,readonly=on,file=/usr/share/ovmf/x64/OVMF_CODE.fd",
-		"if=pflash,format=raw,readonly=on,file=/usr/share/ovmf/x64/OVMF_VARS.fd"
-	]
-	pcie_root_ports: List[str] = []
-	pcie_slave_buses: List[str] = []
-	pcie_slave_devices: List[str] = []
-	harddrives: List[Dict[str, int]] = []
-	cdroms: List[str] = []
-	network: List[Dict[str, Union[str, int, bool]]] = []
-	graphics: bool = False
-	service_location: str = '/etc/systemd/system/'
-	config_location: str = '/etc/qemu.d/'
+	namespace :Optional[str] = None
+	cpu :str = "host"
+	enable_kvm :bool = True
+	machine :str = "q35,accel=kvm"
+	devices :List[str] = ["intel-iommu"]
+	chardev :List[str] = []
+	serial :List[str] = []
+	mon :List[str] = []
+	cpu_cores :Optional[int] = min(4, int(SysCommand('nproc').decode().strip())) # Maximum 4 cores by default
+	memory :int = (psutil.virtual_memory().total / 1024 / 1024) / 8 # Split memory into 8 available machines by default
+	drives :List[str] = []
+	# 	"if=pflash,format=raw,readonly=on,file=/usr/share/ovmf/x64/OVMF_CODE.fd",
+	# 	"if=pflash,format=raw,readonly=on,file=/usr/share/ovmf/x64/OVMF_VARS.fd"
+	# ]
+	pcie_root_ports :Optional[List[str]] = None
+	pcie_slave_buses :Optional[List[str]] = None
+	pcie_slave_devices :Optional[List[str]] = None
+	harddrives :Optional[List[Tuple[str, ...]]] = []
+	cdroms :List[str] = []
+	network :List[NetworkInterfaces] = []
+	graphics :bool = False
+	service_location :str = '/etc/systemd/system/'
+	config_location :str = '/etc/qemu.d/'
+	force: Optional[bool] = False
 
 class ResourceLocations(BaseModel):
 	service_location: str = '/etc/systemd/system/'
 	config_location: str = '/etc/qemu.d/'
 	run_as: str = 'root'
 
+def convert_to_gigabyte(byte :int) -> float:
+	return byte / 1024 / 1024 / 1024
+
 @app.put("/qemu/machine/{name}", tags=["qemu"])
 def create_machine_configuration(name :str, info :NewVirtualMachine, current_user: User = Security(process_user_claim, scopes=["qemu*"])) -> Dict[str, Any]:
 	"""
@@ -49,40 +77,63 @@ def create_machine_configuration(name :str, info :NewVirtualMachine, current_use
 	if not qemu_config_path.parent.exists():
 		qemu_config_path.parent.mkdir(parents=True)
 
-	if service_path.exists():
+	if service_path.exists() and info.force is False:
 		raise HTTPException(status_code=409, detail=f"A service file for the machine {name} already exists: {service_path}")
 
-	if qemu_config_path.exists():
+	if qemu_config_path.exists() and info.force is False:
 		raise HTTPException(status_code=409, detail=f"A environment configuration file for the machine {name} already exists: {qemu_config_path}")
 
 	base_hardware = DupeDict()
 	base_hardware["cpu"] = info.cpu
 	base_hardware["enable-kvm"] = info.enable_kvm
 	base_hardware["machine"] = info.machine
+	base_hardware["m"] = info.memory
+	
 	for device in info.devices:
 		base_hardware["device"] = device
-	base_hardware["m"] = info.memory
 	for drive in info.drives:
 		base_hardware["drive"] = drive
+	for chardev in info.chardev:
+		base_hardware["chardev"] = chardev
+	for serial in info.serial:
+		base_hardware["serial"] = serial
+	for mon in info.mon:
+		base_hardware["mon"] = mon
+	if info.cpu_cores:
+		base_hardware["smp"] = info.cpu_cores
 
 	pcie_buses = DupeDict() # There's already a default pcie.0 bus builtin to qemu.
 
 	pcie_root_ports = DupeDict()
-	for device in info.pcie_root_ports:
-		pcie_root_ports["device"] = device
+	if info.pcie_root_ports:
+		for device in info.pcie_root_ports:
+			pcie_root_ports["device"] = device
 
 	pcie_slave_buses = DupeDict()
-	for device in info.pcie_slave_buses:
-		pcie_slave_buses["device"] = device
+	if info.pcie_slave_buses:
+		for device in info.pcie_slave_buses:
+			pcie_slave_buses["device"] = device
 
 	pcie_slave_devices = DupeDict()
-	for device in info.pcie_slave_devices:
-		pcie_slave_devices["device"] = device
+	if info.pcie_slave_devices:
+		for device in info.pcie_slave_devices:
+			pcie_slave_devices["device"] = device
 
 	scsi_index = 0
 	boot_index = 0
 	if info.harddrives:
 		for drive_index, harddrive_info in enumerate(info.harddrives):
+			if not len(harddrive_info):
+				continue
+			elif len(harddrive_info) == 1 and not len(harddrive_info[0]):
+				continue
+			elif len(harddrive_info) == 2 and not ((harddrive_info[0] and len(harddrive_info[0].strip())) or (harddrive_info[1] and not len(harddrive_info[1].strip()))):
+				continue
+
+			if len(harddrive_info) == 1:
+				disk_info = get_diskimage_information(pathlib.Path(harddrive_info[0]))
+				harddrive_info = tuple([harddrive_info[0], f"{convert_to_gigabyte(disk_info.virtual_size)}G"])
+
 			image_name, size = harddrive_info
 			image_format = pathlib.Path(image_name).suffix[1:]
 			image_path = pathlib.Path(image_name).expanduser().absolute()
@@ -100,6 +151,9 @@ def create_machine_configuration(name :str, info :NewVirtualMachine, current_use
 
 	if info.cdroms:
 		for drive_index, image_name in enumerate(info.cdroms):
+			if len(image_name) == 0:
+				continue
+
 			image_path = pathlib.Path(image_name).expanduser().absolute()
 
 			if not image_path.exists():
@@ -123,10 +177,10 @@ def create_machine_configuration(name :str, info :NewVirtualMachine, current_use
 			"pcie_root_ports": pcie_root_ports,
 			"pcie_slave_buses": pcie_slave_buses,
 			"pcie_slave_devices": pcie_slave_devices,
-			"interfaces": info.network
-		}, cls=JSON))
+			"interfaces": [x.dict(exclude_none=True) for x in info.network] # curate_interfaces(info.network, info.namespace)
+		}, cls=JSON, indent=4))
 
-	write_qemu_service_file(service_path, name, qemu_config_path, namespace=info.namespace)
+	write_qemu_service_file(service_path, name, qemu_config_path, namespace=info.namespace, force=info.force)
 
 	struct = {
 		"name": name,
@@ -143,6 +197,24 @@ def create_machine_configuration(name :str, info :NewVirtualMachine, current_use
 
 	return result
 
+@app.get("/qemu/machine/{name}", tags=["qemu"])
+def get_machine_configuration(name :str, current_user: User = Security(process_user_claim, scopes=["qemu*"])) -> Dict[str, Any]:
+	"""
+	This endpoint starts a qemu instance using a previously created service file with the given name.
+	"""
+
+	qemu_config_path = (pathlib.Path('/etc/qemu.d') / f"{name}.cfg").expanduser().absolute()
+
+	if not qemu_config_path.parent.exists():
+		raise HTTPException(status_code=404, detail="Machine not found")
+
+	with qemu_config_path.open('r') as config:
+		try:
+			conf = json.load(config)
+		except:
+			conf = {}
+
+	return {str(key): val for key,val in conf.items()}
 
 @app.put("/qemu/machine/{name}/string", tags=["qemu"])
 def create_qemu_string_from_struct(name :str, info :NewVirtualMachine, current_user: User = Security(process_user_claim, scopes=["qemu*"])) -> str:
@@ -164,16 +236,19 @@ def create_qemu_string_from_struct(name :str, info :NewVirtualMachine, current_u
 	pcie_buses = DupeDict() # There's already a default pcie.0 bus builtin to qemu.
 
 	pcie_root_ports = DupeDict()
-	for device in info.pcie_root_ports:
-		pcie_root_ports["device"] = device
+	if info.pcie_root_ports:
+		for device in info.pcie_root_ports:
+			pcie_root_ports["device"] = device
 
 	pcie_slave_buses = DupeDict()
-	for device in info.pcie_slave_buses:
-		pcie_slave_buses["device"] = device
+	if info.pcie_slave_buses:
+		for device in info.pcie_slave_buses:
+			pcie_slave_buses["device"] = device
 
 	pcie_slave_devices = DupeDict()
-	for device in info.pcie_slave_devices:
-		pcie_slave_devices["device"] = device
+	if info.pcie_slave_devices:
+		for device in info.pcie_slave_devices:
+			pcie_slave_devices["device"] = device
 
 	return create_qemu_string(name, info.namespace, base_hardware, pcie_buses, pcie_root_ports, pcie_slave_buses, pcie_slave_devices)
 
@@ -206,7 +281,7 @@ def status_qemu_machine(name :str, current_user: User = Security(process_user_cl
 	If the machine is missing or dead, it will return inactive.
 	"""
 
-	return {'status': str(SysCommand(f"systemctl -o json is-active postgresql {name}.service").decode())}
+	return {'status': str(SysCommand(f"systemctl -o json is-active {name}.service").decode().strip())}
 
 @app.patch("/qemu/machine/{name}/snapshot", tags=["qemu"])
 def snapshot_qemu_machine(name :str, current_user: User = Security(process_user_claim, scopes=["qemu*"])) -> None:
@@ -233,4 +308,60 @@ def export_qemu_machine(name :str, current_user: User = Security(process_user_cl
 	This endpoint enables you to export either the configuration or to do a full export including devices.
 	"""
 
-	raise HTTPException(status_code=501, detail="Not yet implemented")
\ No newline at end of file
+	raise HTTPException(status_code=501, detail="Not yet implemented")
+
+@app.get("/qemu/machines", tags=["qemu"])
+def get_all_qemu_machines(current_user: User = Security(process_user_claim, scopes=["qemu*"])) -> Iterator[Dict[str, Any]]:
+	"""
+	This endpoint starts a qemu instance using a previously created service file with the given name.
+	"""
+	for conf_file in glob.glob("/etc/qemu.d/*.cfg"):
+		with open(conf_file) as fh:
+			try:
+				data = json.load(fh)
+			except:
+				log(f"Could not load JSON configuration found at: {conf_file}")
+				continue
+
+			yield {'name': data.get('name', pathlib.Path(conf_file).name[:-4]), **{str(key): val for key, val in data.items()}}
+			# yield data.get('name', pathlib.Path(conf_file).name[:-4])
+
+@app.delete("/qemu/machine/{name}", tags=["qemu"])
+def delete_qemu_machine(name :str, locations :ResourceLocations, current_user: User = Security(process_user_claim, scopes=["qemu*"])) -> None:
+	service_path = (pathlib.Path(locations.service_location) / f"{name}.service").expanduser().absolute()
+	qemu_config_path = (pathlib.Path(locations.config_location) / f"{name}.cfg").expanduser().absolute()
+
+	if not qemu_config_path.parent.exists():
+		qemu_config_path.parent.mkdir(parents=True)
+
+	# TODO: Parse all other *.cfg files and if any drive
+	# is shared, do not unlink them.
+
+	# Read the configuration and extract all drive=X
+	# and unlink the virual drives.
+	with qemu_config_path.open('r') as config:
+		try:
+			conf = json.load(config)
+		except:
+			conf = {}
+
+	for device_type, value_string in conf.get('pcie_slave_devices', []):
+		if 'media=cdrom' in value_string:
+			continue
+
+		if len(device_info_str := re.findall('file=.*?,|format=.*?,', value_string)) == 2:
+			device_info_list = [x.split('=', 1) for x in device_info_str]
+			device_info :Dict[Any, Any] = {}
+			for item in device_info_list:
+				device_info[item[0]] = item[1]
+			
+			if device_info.get('file'):
+				device_location = pathlib.Path(device_info['file'].strip(', '))
+				if device_location.exists():
+					device_location.unlink()
+
+	if service_path.exists():
+		service_path.unlink()
+
+	if qemu_config_path.exists():
+		qemu_config_path.unlink()
\ No newline at end of file
