@@ -14,8 +14,11 @@ from .net import get_interface_info as get_interface_info
 from .net import create_interface as create_interface
 from .net import load_network_info as load_network_info
 from .net import unload_network_info as unload_network_info
+from .net import get_namespaces as get_namespaces
 from .interfaces import generate_mac as generate_mac
 from .interfaces import get_bridge_interfaces as get_bridge_interfaces
+from .interfaces import get_all_mac_addresses_in_use as get_all_mac_addresses_in_use
+from .interfaces import curate_interfaces as curate_interfaces
 
 __all__ = [
 	"epoll",
@@ -35,5 +38,8 @@ __all__ = [
 	"load_network_info",
 	"unload_network_info",
 	"generate_mac",
-	"get_bridge_interfaces"
+	"get_bridge_interfaces",
+	"get_all_mac_addresses_in_use",
+	"get_namespaces",
+	"curate_interfaces"
 ]
\ No newline at end of file
